##########################################################################
##########################################################################
### Application of Specification Testing in RC Models ####################
##########################################################################
##########################################################################

# load relevant packages
library(MASS)
library(expm)
library(psych)
library(EQL)
library(SuppDists)

### Set parameters   #######
imag <- complex(imaginary = 1)

K <- 100  #Number of grid points to approximate the integral
c.vec <- normOrder(K)  #Generate standard normal weights


############################
### Read DATA        #######
############################
Data <- read.csv("HHM_data.csv")   #read data

Y <- Data$Y1      #budget share for food at home
X <- Data$log_X   # log total expenditure
W1 <- Data$W1     # first principal component
W2 <- Data$W2     # second principal component
T1 <- Data$T      # year dummy

# Use subset of data with years 2008 and 2009
t.c <- 18   #for year 2008
t.up <- 19  #for year 2009
Y <- subset(Y, Data$T>= t.c & Data$T<= t.up)
X <- subset(X, Data$T>= t.c & Data$T<= t.up)
W1 <- subset(W1, Data$T>= t.c & Data$T<= t.up)
W2 <- subset(W2, Data$T>= t.c & Data$T<= t.up)
T1 <- subset(T1, Data$T>= t.c & Data$T<= t.up)

# Provide Summary Statistics
summary(Y)
summary(X)

#For the test use Hermite Functions which are global basis functions
#Normalize the data as follows
X  <- X-mean(X)  #demean covariates 
Y0 <- 4*Y        #Enlarges the variance of RCs and thus enables to estimate them with Hermite functions
X <- cbind(X, X^2)
# 
n <- length(Y)
# ones<-rep(1,1,n)


##########################################
##########################################
### Testing Linerity #####################
##########################################
##########################################

## set parameters
k1 <- 4
k2 <- 3
k_n <- (k1+1)+(k2+1) #choice of k_n minimizes test statistic over all k_n<2*n^.25
m_n <- 11  #choice of m_n maximizes the test statistic over all k_n<m_n<sqrt(n)


b.vec <- lm(Y0~X[,1]+W1+W2)$coefficients
Y <- Y0-(b.vec[1]+b.vec[3]*W1+b.vec[4]*W2)


BasX.mat<- mat.or.vec(n,m_n)
for (i in (1:n)){
  BasX.mat[i,]<- pi^{-.25}*exp(-(X[i,1])^2/2)*hermite(X[i,1], 0:(m_n-1), FALSE)/sqrt(2^(0:(m_n-1))*factorial(0:(m_n-1)))
}

## Estimate conditional characteristic function
HX.mat <- crossprod(t(BasX.mat),tcrossprod(ginv(crossprod(BasX.mat)),BasX.mat))
phi.mat <- array(0,c(n,K))
for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }

####### Hermite Eigenfunctions
eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}

Bas.mat<- mat.or.vec(n,k_n)
Bas.f <- function(t){
  for (i in (1:n)){
    Bas.mat[i,] <- c(eval.bas(t,0:k1),eval.bas(t*X[i,1],0:k2))}  
  Bas.mat}

Bas.f.ar <- array(NA, c(n, k_n,K))
for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}

#### Sieve Minimum Distance Estimation   ####
ff0<-function(x){
  A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
  return(sum(abs(A^2)))}

start.vec<-rep(0,k_n)

dens.coef <- optim(start.vec, ff0)$par


U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))

crit.ar <- phi.mat -U.mat

S0.n <- sum(abs(crit.ar)^2)/(K*n)    # Statistic S_n


#######################
#### Critical Values ##
#######################
BasXo.mat <- tcrossprod(sqrtm(ginv(crossprod(BasX.mat))),BasX.mat)


for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }

Bas2.f.ar <- array(NA, c(n, k_n,K))
for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))

crit2.ar<- crit.ar
for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }

mu.vec<- vector()
for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
mu <- sum(mu.vec)/K

eps.vec <- vector()
for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }


###### Now the test statistic is given by 
Test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
print(Test0)
print(1-pnorm(Test0))  # one sided p-value



##########################################
##########################################
### Testing RC QUAIDS ####################
##########################################
##########################################

#set parameters
k1 <- 1
k13 <- 3
k12 <- 2
k_n <- (k1+1)+(k13+1)+(k12+1) #choice of k_n minimizes test statistic over all k_n<2*n^.25
m_n <- 11   #choice of m_n maximizes the test statistic over all k_n<m_n<sqrt(n)

b.vec <- lm(Y0~X[,1]+X[,2]+W1+W2)$coefficients
Y <- Y0-(b.vec[1]+b.vec[4]*W1+b.vec[5]*W2)


  BasX.mat<- mat.or.vec(n,m_n)  ###matrix of Hermite Functions####
  for (i in (1:n)){
    BasX.mat[i,]<- pi^{-.25}*exp(-(X[i,1])^2/2)*hermite(X[i,1], 0:(m_n-1), FALSE)/sqrt(2^(0:(m_n-1))*factorial(0:(m_n-1)))
  }
  
  ## Estimate conditional characteristic function 
  HX.mat <- crossprod(t(BasX.mat),tcrossprod(ginv(crossprod(BasX.mat)),BasX.mat))
  phi.mat <- array(0,c(n,K))
  for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  
  eval.eig.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}
  eval.bas <- function(b,l){exp(-b^2/4)*hermite(b, l, FALSE)/(sqrt(factorial(l)*2^(l-.5)))}
  eval.h.bas <- function(t.c, x,l){ sum(eval.bas(c.vec,l)*exp(imag*t.c*x^2*c.vec))/K}
  
  
  Bas.mat<- mat.or.vec(n,k_n)
  Bas.f <- function(t){
    for (i in (1:n)){
      f.h <- function(m.c) {eval.h.bas(t, X[i,1],m.c)}
      {Bas.mat[i,] <- c(eval.eig.bas(t,0:k1),eval.eig.bas(t*X[i,1],0:k13), sapply(0:k12, f.h))}  }
    Bas.mat}
  
  Bas.f.ar <- array(0, c(n, k_n,K))
  for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}
  
  #### Sieve Minimum Distance Estimation   ####
  ff0<-function(x){
    A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
    return(sum(abs(A^2)))}
  
  start.vec<-rep(0,k_n)
  
  dens.coef <-optim(start.vec, ff0)$par
  
  U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))
  
  crit.ar <- phi.mat -U.mat
  
  S0.n <- sum(abs(crit.ar)^2)/(K*n)    # Statistic S_n
  
  #######################
  #### Critical Values ##
  #######################
  BasXo.mat <- tcrossprod(sqrtm(ginv(crossprod(BasX.mat))),BasX.mat)
  
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  Bas2.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
  U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))
  
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }
  
  
  ###### Now the test statistic is given by 
  Test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  
  print(Test0)
  print(1-pnorm(Test0))

  ##################################################################
  ##################################################################
  ### Testing RC QUAIDS with fixed coeff. on quadratic #############
  ##################################################################
  ##################################################################
 
    k1 <- 1
    k2 <- 4
    k_n <- (k1+1)+(k2+1) #choice of k_n minimizes test statistic over all k_n<2*n^.25
    m_n <- 11   #choice of m_n maximizes the test statistic over all k_n<m_n<sqrt(n)
    
    b.vec <- lm(Y0~X[,1]+X[,2]+W1+W2)$coefficients
    Y <- Y0-(b.vec[1]+b.vec[4]*W1+b.vec[5]*W2)
    b2.hat <- b.vec[3]
    
    BasX.mat<- mat.or.vec(n,m_n)  ###matrix of Hermite Functions####
    for (i in (1:n)){
      BasX.mat[i,]<- pi^{-.25}*exp(-(X[i,1])^2/2)*hermite(X[i,1], 0:(m_n-1), FALSE)/sqrt(2^(0:(m_n-1))*factorial(0:(m_n-1)))
    }
    
    ## Estimate conditional characteristic function 
    HX.mat <- crossprod(t(BasX.mat),tcrossprod(ginv(crossprod(BasX.mat)),BasX.mat))
    phi.mat <- array(0,c(n,K))
    for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
    
    
    ####### Hermite Eigenfunctions
    eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}
    
    Bas.mat<- mat.or.vec(n,k_n)
    Bas.f <- function(t){
      for (i in (1:n)){
        Bas.mat[i,] <- c(eval.bas(t,0:k1),eval.bas(t*X[i,1],0:k2))}  
      Bas.mat}
    
    Bas.f.ar <- array(NA, c(n, k_n,K))
    for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}
    
    #### Sieve Minimum Distance Estimation   ####
    ff0<-function(x){
      A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) 
        (Bas.f.ar[,,i]*exp(imag*X[,2]*x[k_n+1]*c.vec[i]))%*%x[1:k_n]))
      return(sum(abs(A^2)))}
    
    start.vec<-rep(0,k_n+1)
    dens.coef <- optim(start.vec, ff0)$par
    
    b2.hat<-dens.coef[k_n+1]
    
    U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) 
      (Bas.f.ar[,,i]*exp(imag*X[,2]*b2.hat*c.vec[i]))%*%dens.coef[1:k_n]))
    crit.ar <- phi.mat -U.mat
    
    S0.n <- sum(abs(crit.ar)^2)/(K*n)    # Statistic S_n
    
    #######################
    #### Critical Values ##
    #######################
    BasXo.mat <- tcrossprod(sqrtm(ginv(crossprod(BasX.mat))),BasX.mat)
    for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
    
    Bas2.f.ar <- array(NA, c(n, k_n,K))
    for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
    U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i)
      (Bas2.f.ar[,,i]*exp(imag*X[,2]*b2.hat*c.vec[i]))%*%dens.coef[1:k_n]))
    
    crit2.ar<- crit.ar
    for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
    
    mu.vec<- vector()
    for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
    mu <- sum(mu.vec)/K
    
    eps.vec <- vector()
    for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }
    
    
    ###### Now the test statistic is given by 
    Test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
    print(Test0)
    print(1-pnorm(Test0))
  

########################################    
########################################
#####   Control Function Residuals #####
########################################
########################################
    
# Read additional DATA
    Z <- Data$log_Z   #labor income Z
    Z <- subset(Z, Data$T>= t.c & Data$T<= t.up)
    
    b.vec <- lm(Y0~X[,1]+W1+W2)$coefficients
    Y <- Y0-(b.vec[1]+b.vec[3]*W1+b.vec[4]*W2)

# load additional packages
library(np)
library(crs)


BasZ.mat <- gsl.bs(Z,degree=3, nbreak=5)
BasZ.mat <-cbind(1,BasZ.mat)


HZ.mat <- BasZ.mat%*%ginv(t(BasZ.mat)%*%BasZ.mat)%*%t(BasZ.mat)
dist.est <- function(x){ crossprod(t(HZ.mat),as.numeric(X[,1]<=x)) }
A <- mat.or.vec(n,n)
for ( i in 1:n)
{ A[i,] <-   dist.est(X[i,1])   }

V <- diag(A)
dens.V <-npudens(V)$dens    #density of control function residuals

### Testing Linerity #####################

# Set dimension parameters
k1<-1
k2<-3
k_v<-1
k_n<-(1+k1)+(1+k2)+(1+k_v) #choice of k_n minimizes test statistic over all k_n<2*n^.25
m <- 5
mV <- 1  #choice of m_n=2+m+mV maximizes the test statistic over all k_n<m_n<sqrt(n)


BasX.mat<- mat.or.vec(n,m+1)
for (i in (1:n)){
  BasX.mat[i,]<- pi^{-.25}*exp(-(X[i,1])^2/2)*hermite(X[i,1], 0:m, FALSE)/sqrt(2^(0:m)*factorial(0:m))
}

BasV.mat<- mat.or.vec(n,mV+1)
for (i in (1:n)){
  BasV.mat[i,]<- pi^{-.25}*exp(-(V[i])^2/2)*hermite(V[i], 0:mV, FALSE)/sqrt(2^(0:mV)*factorial(0:mV))
}

BasXV.mat <- mat.or.vec(n,(length(BasX.mat[1,])*length(BasV.mat[1,])))
for( i in 1:n){ BasXV.mat[i,] <- kronecker(BasX.mat[i,],BasV.mat[i,])}

## Estimate conditional characteristic function 
HX.mat <- crossprod(t(BasXV.mat),tcrossprod(ginv(crossprod(BasXV.mat)),BasXV.mat))
phi.mat <- array(0,c(n,K))
for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }

####### Hermite Eigenfunctions
eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}

Bas.mat<- mat.or.vec(n,k_n)
Bas.f <- function(t){
  for (i in (1:n)){
    Bas.mat[i,] <- c(eval.bas(t,0:k1),c(eval.bas(t*X[i,1],0:k2), eval.bas(t*V[i],0:k_v)/(dens.V[i]) ))}
  Bas.mat}

Bas.f.ar <- array(NA, c(n, k_n,K))
for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}

#### Sieve Minimum Distance Estimation   ####
ff0<-function(x){
  A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
  return(sum(abs(A^2)))}

start.vec<-rep(0,k_n)

dens.coef <- optim(start.vec, ff0)$par


U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))

crit.ar <- phi.mat -U.mat

S0.n <- sum(abs(crit.ar)^2)/(K*n)

#######################
#### Critical Values ##
#######################
BasXo.mat <- sqrtm(ginv(t(BasXV.mat)%*%BasXV.mat))%*%t(BasXV.mat)

for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }  #-U.mat[,c.k]

Bas2.f.ar <- array(NA, c(n, k_n,K))
for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))

crit2.ar<- crit.ar
for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }

mu.vec<- vector()
for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
mu <- sum(mu.vec)/K

eps.vec <- vector()
for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }


###### Now the test statistic is given by
Test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))

print(Test0)
print(1-pnorm(Test0))

########################################
##### Testing RC Quaids         ########
##### control function residuals  ######
########################################
  k1 <- 1
  k13 <- 3
  k12 <- 1
  k1v <- 1
  k_n <- (k13+1)+(k1+1)+(k12+1)+(k1v+1) #choice of k_n minimizes test statistic over all k_n<2*n^.25
  m<-5
  mV<-2 #choice of m_n=2+m+mV maximizes the test statistic over all k_n<m_n<sqrt(n)
  
  b.vec <- lm(Y0~X[,1]+X[,2]+W1+W2)$coefficients
  Y <- Y0-(b.vec[1]+b.vec[4]*W1+b.vec[5]*W2)

  BasX.mat<- mat.or.vec(n,m+1)
  for (i in (1:n)){
    BasX.mat[i,]<- pi^{-.25}*exp(-(X[i,1])^2/2)*hermite(X[i,1], 0:m, FALSE)/sqrt(2^(0:m)*factorial(0:m))
  }
  
  BasV.mat<- mat.or.vec(n,mV+1)
  for (i in (1:n)){
    BasV.mat[i,]<- pi^{-.25}*exp(-(V[i])^2/2)*hermite(V[i], 0:mV, FALSE)/sqrt(2^(0:mV)*factorial(0:mV))
  }
  
  BasXV.mat <- mat.or.vec(n,(length(BasX.mat[1,])*length(BasV.mat[1,])))
  for( i in 1:n){ BasXV.mat[i,] <- kronecker(BasX.mat[i,],BasV.mat[i,])}
  
  ## Estimate conditional characteristic function 
  HX.mat <- crossprod(t(BasXV.mat),tcrossprod(ginv(crossprod(BasXV.mat)),BasXV.mat))
  phi.mat <- array(0,c(n,K))
  for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }

  ####### Hermite Eigenfunctions
  eval.bas <- function(b,l){exp(-b^2/4)*hermite(b, l, FALSE)/(sqrt(factorial(l)*2^(l-.5)))}
  eval.h.bas <- function(t.c, x,l){ sum(eval.bas(c.vec,l)*exp(imag*t.c*x^2*c.vec))/K}
  
  Bas.mat<- mat.or.vec(n,k_n)
  Bas.f <- function(t){
    for (i in (1:n)){
      f.h <- function(m.c) {eval.h.bas(t, X[i,1],m.c)}
      {Bas.mat[i,] <- c(eval.eig.bas(t,0:k1),c(c(eval.eig.bas(t*X[i,1],0:k13), sapply(0:k12, f.h)), eval.bas(t*V[i],0:k1v)/(dens.V[i])))}  }
    Bas.mat}
  
  Bas.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}
  
  #### Sieve Minimum Distance Estimation   ####
  ff0<-function(x){
    A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%x))
    return(sum(abs(A^2)))}
  
  start.vec<-rep(0,k_n)
  dens.coef <- optim(start.vec, ff0)$par
  
  U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) Bas.f.ar[,,i]%*%dens.coef))
  
  crit.ar <- phi.mat -U.mat
  
  S0.n <- sum(abs(crit.ar)^2)/(K*n)
  #######################
  #### Critical Values ##
  #######################
  BasXo.mat <- sqrtm(ginv(t(BasXV.mat)%*%BasXV.mat))%*%t(BasXV.mat)
  
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  Bas2.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
  U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i) Bas2.f.ar[,,i]%*%dens.coef))
  
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }
  
  
  ###### Now the test statistic is given by 
  Test0 <- (n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K))
  
  print(Test0)
  print(1-pnorm(Test0))


#################################################################
#####  Testing RC QUAIDS with fixed coeff. on quadratic #########
#####   control function residuals          #####################
#################################################################
  k1<-1
  k2<-4
  k_v<-4
  k_n<-(1+k1)+(1+k2)+(1+k_v) #choice of k_n minimizes test statistic over all k_n<2*n^.25
  m <- 5
  m_v<-5 #choice of m_n=2+m+mV maximizes the test statistic over all k_n<m_n<sqrt(n)
  
  
  b.vec <- lm(Y0~X[,1]+X[,2]+W1+W2)$coefficients
  Y <- Y0-(b.vec[1]+b.vec[4]*W1+b.vec[5]*W2)
  
  BasX.mat<- mat.or.vec(n,m+1)
  for (i in (1:n)){
    BasX.mat[i,]<- pi^{-.25}*exp(-(X[i,1])^2/2)*hermite(X[i,1], 0:m, FALSE)/sqrt(2^(0:m)*factorial(0:m))
  }
  
  BasV.mat<- mat.or.vec(n,m_v+1)
  for (i in (1:n)){
    BasV.mat[i,]<- pi^{-.25}*exp(-(V[i])^2/2)*hermite(V[i], 0:m_v, FALSE)/sqrt(2^(0:m_v)*factorial(0:m_v))
  }
  
  
  BasXV.mat <- mat.or.vec(n,(length(BasX.mat[1,])*length(BasV.mat[1,])))
  for( i in 1:n){ BasXV.mat[i,] <- kronecker(BasX.mat[i,],BasV.mat[i,])}
  
  ## Estimate conditional characteristic function 
  HX.mat <- crossprod(t(BasXV.mat),tcrossprod(ginv(crossprod(BasXV.mat)),BasXV.mat))
  phi.mat <- array(0,c(n,K))
  for(c.k in 1:K) {phi.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }

  ####### Hermite Eigenfunctions
  eval.bas <- function(x,l){sqrt(2*pi)*pi^{-.25}*exp(-x^2/2)*hermite(x, l, FALSE)/((-imag)^(l)*sqrt(2^(l)*factorial(l)))}
  
  Bas.mat<- mat.or.vec(n,k_n)
  Bas.f <- function(t){
    for (i in (1:n)){
      Bas.mat[i,] <- c(eval.bas(t,0:k1),c(eval.bas(t*X[i,1],0:k2), eval.bas(t*V[i],0:k_v)/(dens.V[i]) ))}  
    Bas.mat}
  
  Bas.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas.f.ar[,,i] <- Bas.f(c.vec[i])}
  
#### Sieve Minimum Distance Estimation   ####
  ff0<-function(x){
    A<- phi.mat-do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) 
      (Bas.f.ar[,,i]*exp(imag*X[,2]*x[k_n+1]*c.vec[i]))%*%x[1:k_n]))
    return(sum(abs(A^2)))}
  
  start.vec<-rep(0,k_n+1)
  
  dens.coef <- optim(start.vec, ff0)$par
  
  b2.hat<-dens.coef[k_n+1]
  
  U.mat <- do.call(cbind,lapply(seq_len(dim(Bas.f.ar)[3]),function(i) 
    (Bas.f.ar[,,i]*exp(imag*X[,2]*b2.hat*c.vec[i]))%*%dens.coef[1:k_n]))
  
  crit.ar <- phi.mat -U.mat
  
  S0.n <- sum(abs(crit.ar)^2)/(K*n)
  #######################
  #### Critical Values ##
  #######################
  BasXo.mat <- sqrtm(ginv(t(BasXV.mat)%*%BasXV.mat))%*%t(BasXV.mat)
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  Bas2.f.ar <- array(NA, c(n, k_n,K))
  for(i in 1:K){Bas2.f.ar[,,i] <- Bas.f(c.vec[i])}
  U2.mat <- do.call(cbind,lapply(seq_len(dim(Bas2.f.ar)[3]),function(i)
    (Bas2.f.ar[,,i]*exp(imag*X[,2]*b2.hat*c.vec[i]))%*%dens.coef[1:k_n]))
  
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {mu.vec[c.k] <- tr(BasXo.mat%*%diag(as.vector(abs(crit.ar[,c.k] )^2))%*%t(BasXo.mat))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(t(BasXo.mat*crit.ar[,c.k])%*%(BasXo.mat*crit2.ar[,c.k])))^2) }
  
  
  ###### Now the test statistic is given by 
  Test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  
  print(Test0)
  print(1-pnorm(Test0))
