###########################################################
###  Testing Degeneracy ###################################
###########################################################

# load relevant packages
library(MASS)
library(expm)
library(parallel)
library(crs)

############################
### Set parameters   #######
############################

n <- 500   #Sample Size
N <- 1000  #Number of Monte Carlo Iterations

core.c <- 1  #Number of cores used for parallel computing
              #If using windows then set core.c <- 1

imag <- complex( imaginary = 1)

inter.const <- 1 #  or 1.5, corresponds to the constant \rho in Section 3.2

ones<-rep(1,1,n)

K <- 50    #Number of grid points to approximate the integral
c.vec <- normOrder(K)  # Set vector of normal weights

####################################
### Set Tuning parameters   #######
####################################

m2_n <- 4  # or 5

k_n <- 4  # or 5

# the dimension parameter m_n satisfies m_n=k_n*m2_n, so m_n is either 16, 20 or 25

##################################################
####### MONTE CARLO SIMULATION####################
################################################## 
test=matrix(nrow=N,ncol=5)

MC.fct <- function(MC)
{
  ######### Generate DATA   #############
  X <- mvrnorm(n, rep(0, 2), matrix(c(1,.5,.5,1),2,2), empirical = FALSE, EISPACK = FALSE) 
  B<- mvrnorm(n, c(0,0), matrix(c(2,inter.const,inter.const,2),2,2), empirical = FALSE, EISPACK = FALSE)
  B1 <- B[,1]
  B2 <- B[,2]
  ############################
  ### Null Hypothesis  #######
  ############################
  B1 <- B[,1]
  B2 <- 1 ######## 2nd RC is degenerate
  B0 <- B[,2]/4
  
  
  Y <- B0 + B1*X[,1]+B2*X[,2] ####### Generate Y under H_deg
  
  
  ######### Estimate the degenerate RC as described in Example 4 in the paper
  b.vec <- lm(Y~X[,1]+X[,2])$coefficients
  b.hat <- b.vec[3]
  
  Un <- function(x){exp(imag*x*(Y-b.hat*X[,2]))}
  
  ######### ######### ######### ######### ######### ######### ######### #########
  ##### 1. We estimate the constraint estimator using degeneracy of the 2nd RC
  ######### ######### ######### ######### ######### ######### ######### #########
  BasXh.mat <- gsl.bs(X[,1],degree=2, nbreak=(k_n-1)) 
  BasX1.mat <- cbind(ones,BasXh.mat)     ####### this is the matrix X_{1n} (see also Example 4)
  
  HX1.mat <- crossprod(t(BasX1.mat),tcrossprod(ginv(crossprod(BasX1.mat)),BasX1.mat))
  U.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k]))*exp(imag*c.vec[c.k]*b.hat*X[,2])

  
  BasXh.mat <- gsl.bs(X[,2],degree=2, nbreak=(m2_n-1))
  BasX2.mat <- cbind(ones,BasXh.mat)
  
 
  BasX.mat <- mat.or.vec(n,(length(BasX1.mat[1,])*length(BasX2.mat[1,])))
  for( i in 1:n){ BasX.mat[i,] <- kronecker(BasX1.mat[i,],BasX2.mat[i,])}
  
  
  HX.mat <- crossprod(t(BasX.mat),tcrossprod(ginv(crossprod(BasX.mat)),BasX.mat))
  crit.mat <- array(0,c(n,K))
  for(c.k in 1:K) {crit.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  
  crit.ar <- crit.mat -U.mat
  S0.n <- sum(abs(crit.ar)^2)/(K*n)
  
  ###############################################################
  #### Estimation of mean and variance for standardization ######
  ###############################################################
  
  BasXo.mat <- tcrossprod(sqrtm(ginv(crossprod(BasX.mat))),BasX.mat)
  
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }  #-U.mat[,c.k]
  
  U2.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k])*exp(imag*c.vec[c.k]*b.hat*X[,2]))
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {
    mu.vec[c.k] <-sum(diag(crossprod(t(BasXo.mat),tcrossprod(diag(as.vector(abs(crit.ar[,c.k] )^2)),BasXo.mat))))}
  mu <- sum(mu.vec)/K
    
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(crossprod(BasXo.mat*crit.ar[,c.k],(BasXo.mat*crit2.ar[,c.k]))))^2) }
  
  test0 <- Re((n*S0.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  
  
  ##################
  #1st Alternative
  ##################
  B1 <- B[,1]
  B2 <- 0.3*B[,2]       ##### Now the 2nd coefficient is random
  c<- var(B0 +B1*X[,1]+X[,2])/var(B0 + B1*X[,1]+B2*X[,2])
  Y <- (B0 + B1*X[,1]+B2*X[,2])*sqrt(c)
  b.vec <- lm(Y~X[,1]+X[,2])$coefficients
  b.hat <- b.vec[3]
  
  Un <- function(x){exp(imag*x*(Y-b.hat*X[,2]))}
  U.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k]))*exp(imag*c.vec[c.k]*b.hat*X[,2])
  
  crit.mat <- array(0,c(n,K))
  for(c.k in 1:K) {crit.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  crit.ar <- crit.mat -U.mat
  
  S1.n <- sum(abs(crit.ar)^2)/(K*n)
  
  ###############################################################
  #### Estimation of mean and variance for standardization ######
  ###############################################################
  
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }  #-U.mat[,c.k]
  
  U2.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k])*exp(imag*c.vec[c.k]*b.hat*X[,2]))
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {
    mu.vec[c.k] <-sum(diag(crossprod(t(BasXo.mat),tcrossprod(diag(as.vector(abs(crit.ar[,c.k] )^2)),BasXo.mat))))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(crossprod(BasXo.mat*crit.ar[,c.k],(BasXo.mat*crit2.ar[,c.k]))))^2) }
  
  test1 <- Re((n*S1.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  
  
  ##################
  #2nd Alternative
  ##################
  B1 <- B[,1]
  B2 <- .5*B[,2]       ##### Now the 2nd coefficient is random
  c<- var(B0 +B1*X[,1]+X[,2])/var(B0 + B1*X[,1]+B2*X[,2])
  Y <- (B0 + B1*X[,1]+B2*X[,2])*sqrt(c)
  b.vec <- lm(Y~X[,1]+X[,2])$coefficients
  b.hat <- b.vec[3]
  
  Un <- function(x){exp(imag*x*(Y-b.hat*X[,2]))}
 
  U.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k]))*exp(imag*c.vec[c.k]*b.hat*X[,2])
  
  
   
  crit.mat <- array(0,c(n,K))
  for(c.k in 1:K) {crit.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  crit.ar <- crit.mat -U.mat
  
  S1.n <- sum(abs(crit.ar)^2)/(K*n)
  
  ###############################################################
  #### Estimation of mean and variance for standardization ######
  ###############################################################
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }  #-U.mat[,c.k]
  
  U2.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k]))*exp(imag*c.vec[c.k]*b.hat*X[,2])
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {
    mu.vec[c.k] <-sum(diag(crossprod(t(BasXo.mat),tcrossprod(diag(as.vector(abs(crit.ar[,c.k] )^2)),BasXo.mat))))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(crossprod(BasXo.mat*crit.ar[,c.k],(BasXo.mat*crit2.ar[,c.k]))))^2) }

  test2 <- Re((n*S1.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  
  
  
  ##################
  #3rd Alternative
  ##################
  B1 <- B[,1]
  B2 <- .7*B[,2]       ##### Now the 2nd coefficient is random
  c<- var(B0 +B1*X[,1]+X[,2])/var(B0 + B1*X[,1]+B2*X[,2])
  Y <- (B0 + B1*X[,1]+B2*X[,2])*sqrt(c)
  b.vec <- lm(Y~X[,1]+X[,2])$coefficients
  b.hat <- b.vec[3]
  
  Un <- function(x){exp(imag*x*(Y-b.hat*X[,2]))}
  
  U.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k]))*exp(imag*c.vec[c.k]*b.hat*X[,2])
  
  crit.mat <- array(0,c(n,K))
  for(c.k in 1:K) {crit.mat[,c.k] <- crossprod(t(HX.mat),(exp(imag*Y*c.vec[c.k]))) }
  crit.ar <- crit.mat -U.mat
  
  S1.n <- sum(abs(crit.ar)^2)/(K*n)
 
  ###############################################################
  #### Estimation of mean and variance for standardization ######
  ###############################################################
  for(c.k in 1:K) {crit.ar[,c.k] <- exp(imag*Y*c.vec[c.k])-U.mat[,c.k] }
  
  U2.mat <- array(0,c(n,K))
  for(c.k in 1:K) U.mat[,c.k] <- crossprod(HX1.mat,Un(c.vec[c.k])*exp(imag*c.vec[c.k]*b.hat*X[,2]))
  crit2.ar<- crit.ar
  for(c.k in 1:K) {crit2.ar[,c.k] <- Conj(exp(imag*Y*c.vec[c.k])-U2.mat[,c.k]) }
  
  mu.vec<- vector()
  for(c.k in 1:K) {
    mu.vec[c.k] <-sum(diag(crossprod(t(BasXo.mat),tcrossprod(diag(as.vector(abs(crit.ar[,c.k] )^2)),BasXo.mat))))}
  mu <- sum(mu.vec)/K
  
  eps.vec <- vector()
  for(c.k in 1:K) { eps.vec[c.k] <- sum((abs(crossprod(BasXo.mat*crit.ar[,c.k],(BasXo.mat*crit2.ar[,c.k]))))^2) }

  test3 <- Re((n*S1.n - mu)/(sqrt(2)*sqrt(sum(eps.vec)/K)))
  
  
  c(test0, test1, test2, test3)
  
}

test <- t(mcmapply(MC.fct,1:N, mc.cores = core.c))

#### Compute Empirical Rejection Probabilities at nominal level 0.05
Test.Matrix <- matrix(ncol=4,nrow=N)
for (l in 1:4){  for (MC in 1:N){ Test.Matrix[MC,l]=as.integer(test[MC,l]>=qnorm(0.95))}}

print(colMeans(Test.Matrix))

